export type LogLevelId = 'silent' | 'error' | 'warn' | 'debug';
export declare function debug(logLevel: LogLevelId, ...messages: any[]): void;
export declare function warn(logLevel: LogLevelId, warning: string | Error): void;
